@@ -42,42 +42,47 @@
 #include "acct/radius.h"
 #include "common-config.h"
 
-static rc_handle *rh = NULL;
-static char nas_identifier[64];
-
-static void acct_radius_global_init(void *pool, void *additional)
+static void acct_radius_vhost_init(void **_vctx, void *pool, void *additional)
 {
 	radius_cfg_st *config = additional;
+	struct radius_vhost_ctx *vctx;
 
 	if (config == NULL)
 		goto fail;
 
-	rh = rc_read_config(config->config);
-	if (rh == NULL)
+	vctx = talloc_zero(pool, struct radius_vhost_ctx);
+	if (vctx == NULL)
 		goto fail;
 
-	if (config->nas_identifier) {
-		strlcpy(nas_identifier, config->nas_identifier, sizeof(nas_identifier));
-	} else {
-		nas_identifier[0] = 0;
+	vctx->rh = rc_read_config(config->config);
+	if (vctx->rh == NULL) {
+		goto fail;
 	}
 
-	if (rc_read_dictionary(rh, rc_conf_str(rh, "dictionary")) != 0) {
+	if (config->nas_identifier) {
+		strlcpy(vctx->nas_identifier, config->nas_identifier, sizeof(vctx->nas_identifier));
+	} else {
+		vctx->nas_identifier[0] = 0;
+	}
+
+	if (rc_read_dictionary(vctx->rh, rc_conf_str(vctx->rh, "dictionary")) != 0) {
 		fprintf(stderr, "error reading the radius dictionary\n");
 		exit(1);
 	}
+	*_vctx = vctx;
 
 	return;
  fail:
- 	fprintf(stderr, "radius acct initialization error\n");
+	fprintf(stderr, "radius initialization error\n");
 	exit(1);
-
 }
 
-static void acct_radius_global_deinit(void)
+static void acct_radius_vhost_deinit(void *_vctx)
 {
-	if (rh != NULL)
-		rc_destroy(rh);
+	struct radius_vhost_ctx *vctx = _vctx;
+
+	if (vctx->rh != NULL)
+		rc_destroy(vctx->rh);
 }
 
 static void append_stats(rc_handle *rh, VALUE_PAIR **send, stats_st *stats)
@@ -104,12 +109,12 @@ static void append_stats(rc_handle *rh, VALUE_PAIR **send, stats_st *stats)
 	return;
 }
 
-static void append_acct_standard(rc_handle *rh, const common_acct_info_st *ai, VALUE_PAIR **send)
+static void append_acct_standard(struct radius_vhost_ctx *vctx, rc_handle *rh, const common_acct_info_st *ai, VALUE_PAIR **send)
 {
 	uint32_t i;
 
-	if (nas_identifier[0] != 0) {
-		rc_avpair_add(rh, send, PW_NAS_IDENTIFIER, nas_identifier, -1, 0);
+	if (vctx->nas_identifier[0] != 0) {
+		rc_avpair_add(rh, send, PW_NAS_IDENTIFIER, vctx->nas_identifier, -1, 0);
 	}
 
 	if (ai->our_ip[0] != 0) {
@@ -162,24 +167,25 @@ static void append_acct_standard(rc_handle *rh, const common_acct_info_st *ai, V
 	return;
 }
 
-static void radius_acct_session_stats(unsigned auth_method, const common_acct_info_st *ai, stats_st *stats)
+static void radius_acct_session_stats(void *_vctx, unsigned auth_method, const common_acct_info_st *ai, stats_st *stats)
 {
-int ret;
-uint32_t status_type;
-VALUE_PAIR *send = NULL, *recvd = NULL;
+	int ret;
+	uint32_t status_type;
+	VALUE_PAIR *send = NULL, *recvd = NULL;
+	struct radius_vhost_ctx *vctx = _vctx;
 
 	status_type = PW_STATUS_ALIVE;
 
 	syslog(LOG_DEBUG, "radius-auth: sending session interim update");
 
-	if (rc_avpair_add(rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL) {
+	if (rc_avpair_add(vctx->rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL) {
 		goto cleanup;
 	}
 
-	append_acct_standard(rh, ai, &send);
-	append_stats(rh, &send, stats);
+	append_acct_standard(vctx, vctx->rh, ai, &send);
+	append_stats(vctx->rh, &send, stats);
 
-	ret = rc_aaa(rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
+	ret = rc_aaa(vctx->rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
 
 	if (recvd != NULL)
 		rc_avpair_free(recvd);
@@ -194,11 +200,12 @@ VALUE_PAIR *send = NULL, *recvd = NULL;
 	return;
 }
 
-static int radius_acct_open_session(unsigned auth_method, const common_acct_info_st *ai, const void *sid, unsigned sid_size)
+static int radius_acct_open_session(void *_vctx, unsigned auth_method, const common_acct_info_st *ai, const void *sid, unsigned sid_size)
 {
-int ret;
-uint32_t status_type;
-VALUE_PAIR *send = NULL, *recvd = NULL;
+	int ret;
+	uint32_t status_type;
+	VALUE_PAIR *send = NULL, *recvd = NULL;
+	struct radius_vhost_ctx *vctx = _vctx;
 
 	status_type = PW_STATUS_START;
 
@@ -209,18 +216,18 @@ VALUE_PAIR *send = NULL, *recvd = NULL;
 
 	syslog(LOG_DEBUG, "radius-auth: opening session %s", ai->safe_id);
 
-	if (rc_avpair_add(rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL) {
+	if (rc_avpair_add(vctx->rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL) {
 		ret = -1;
 		goto cleanup;
 	}
 
 	if (ai->user_agent[0] != 0) {
-		rc_avpair_add(rh, &send, PW_CONNECT_INFO, ai->user_agent, -1, 0);
+		rc_avpair_add(vctx->rh, &send, PW_CONNECT_INFO, ai->user_agent, -1, 0);
 	}
 
-	append_acct_standard(rh, ai, &send);
+	append_acct_standard(vctx, vctx->rh, ai, &send);
 
-	ret = rc_aaa(rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
+	ret = rc_aaa(vctx->rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
 
 	if (recvd != NULL)
 		rc_avpair_free(recvd);
@@ -237,16 +244,17 @@ VALUE_PAIR *send = NULL, *recvd = NULL;
 	return ret;
 }
 
-static void radius_acct_close_session(unsigned auth_method, const common_acct_info_st *ai, stats_st *stats, unsigned discon_reason)
+static void radius_acct_close_session(void *_vctx, unsigned auth_method, const common_acct_info_st *ai, stats_st *stats, unsigned discon_reason)
 {
-int ret;
-uint32_t status_type;
-VALUE_PAIR *send = NULL, *recvd = NULL;
+	int ret;
+	uint32_t status_type;
+	VALUE_PAIR *send = NULL, *recvd = NULL;
+	struct radius_vhost_ctx *vctx = _vctx;
 
 	status_type = PW_STATUS_STOP;
 
 	syslog(LOG_DEBUG, "radius-auth: closing session");
-	if (rc_avpair_add(rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL)
+	if (rc_avpair_add(vctx->rh, &send, PW_ACCT_STATUS_TYPE, &status_type, -1, 0) == NULL)
 		return;
 
 	if (discon_reason == REASON_USER_DISCONNECT)
@@ -263,12 +271,12 @@ VALUE_PAIR *send = NULL, *recvd = NULL;
 		ret = PW_USER_ERROR;
 	else
 		ret = PW_LOST_SERVICE;
-	rc_avpair_add(rh, &send, PW_ACCT_TERMINATE_CAUSE, &ret, -1, 0);
+	rc_avpair_add(vctx->rh, &send, PW_ACCT_TERMINATE_CAUSE, &ret, -1, 0);
 
-	append_acct_standard(rh, ai, &send);
-	append_stats(rh, &send, stats);
+	append_acct_standard(vctx, vctx->rh, ai, &send);
+	append_stats(vctx->rh, &send, stats);
 
-	ret = rc_aaa(rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
+	ret = rc_aaa(vctx->rh, ai->id, send, &recvd, NULL, 1, PW_ACCOUNTING_REQUEST);
 	if (recvd != NULL)
 		rc_avpair_free(recvd);
 
@@ -285,8 +293,8 @@ VALUE_PAIR *send = NULL, *recvd = NULL;
 const struct acct_mod_st radius_acct_funcs = {
 	.type = ACCT_TYPE_RADIUS,
 	.auth_types = ALL_AUTH_TYPES,
-	.global_init = acct_radius_global_init,
-	.global_deinit = acct_radius_global_deinit,
+	.vhost_init = acct_radius_vhost_init,
+	.vhost_deinit = acct_radius_vhost_deinit,
 	.open_session = radius_acct_open_session,
 	.close_session = radius_acct_close_session,
 	.session_stats = radius_acct_session_stats
