@@ -693,11 +693,26 @@ int secmod_reload(main_server_st * s)
 	return 0;
 }
 
-void secmod_socket_file_name(struct perm_cfg_st *perm_config, char *name, unsigned max_name_size)
+/* Creates a permanent filename to use for secmod to main communication
+ */
+const char *secmod_socket_file_name(struct perm_cfg_st *perm_config)
 {
-	/* make socket name and full socket name */
-	snprintf(name, max_name_size, "%s.%u",
-		 perm_config->socket_file_prefix, (unsigned)getpid());
+	unsigned int rnd;
+	int ret;
+	static char socket_file[_POSIX_PATH_MAX] = {0};
+
+	if (socket_file[0] != 0)
+		return socket_file;
+
+	ret = gnutls_rnd(GNUTLS_RND_NONCE, &rnd, sizeof(rnd));
+	if (ret < 0)
+		exit(1);
+
+	/* make socket name */
+	snprintf(socket_file, sizeof(socket_file), "%s.%x",
+		 perm_config->socket_file_prefix, rnd);
+
+	return socket_file;
 }
 
 static void clear_unneeded_mem(struct list_head *vconfig)
@@ -721,7 +736,9 @@ int run_sec_mod(main_server_st *s, int *sync_fd)
 	pid_t pid;
 	const char *p;
 
-	secmod_socket_file_name(GETPCONFIG(s), s->socket_file, sizeof(s->socket_file));
+	/* fills s->socket_file */
+	strlcpy(s->socket_file, secmod_socket_file_name(GETPCONFIG(s)), sizeof(s->socket_file));
+	mslog(s, NULL, LOG_DEBUG, "created sec-mod socket file (%s)", s->socket_file);
 
 	if (GETPCONFIG(s)->chroot_dir != NULL) {
 		ret = snprintf(s->full_socket_file, sizeof(s->full_socket_file), "%s/%s",
