@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015 Nikos Mavrogiannopoulos
+ * Copyright (C) 2013-2018 Nikos Mavrogiannopoulos
  *
  * This file is part of ocserv.
  *
@@ -36,29 +36,26 @@
 #include <sys/un.h>
 #include <common.h>
 #include <syslog.h>
-#include <vpn.h>
+#include <main.h>
 #include <sec-mod.h>
 #include <tlslib.h>
 #include <ipc.pb-c.h>
 #include <sec-mod-sup-config.h>
 #include <sec-mod-resume.h>
 #include <cloexec.h>
+#include <assert.h>
 
 #include <gnutls/gnutls.h>
 #include <gnutls/crypto.h>
 #include <gnutls/abstract.h>
 
-#define MAX_PIN_SIZE GNUTLS_PKCS11_MAX_PIN_LEN
 #define MAINTAINANCE_TIME 310
 
 static int need_maintainance = 0;
 static int need_reload = 0;
 static int need_exit = 0;
 
-struct pin_st {
-	char pin[MAX_PIN_SIZE];
-	char srk_pin[MAX_PIN_SIZE];
-};
+static void reload_server(sec_mod_st *sec);
 
 static int load_keys(sec_mod_st *sec, unsigned force);
 
@@ -199,14 +196,16 @@ static int handle_op(void *pool, int cfd, sec_mod_st * sec, uint8_t type, uint8_
 }
 
 static
-int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_request_t cmd,
+int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st *sec, cmd_request_t cmd,
 		   uint8_t * buffer, size_t buffer_size)
 {
 	unsigned i;
 	gnutls_datum_t data, out;
 	int ret;
 	SecOpMsg *op;
+	vhost_cfg_st *vhost;
 #if GNUTLS_VERSION_NUMBER >= 0x030600
+	unsigned bits;
 	SecGetPkMsg *pkm;
 #endif
 	PROTOBUF_ALLOCATOR(pa, pool);
@@ -225,14 +224,19 @@ int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_
 			return -1;
 		}
 
+		vhost = find_vhost(sec->vconfig, pkm->vhost);
+		/* check for static analyzer - find_vhost is always non-NULL */
+		assert(vhost != NULL);
+
 		i = pkm->key_idx;
-		if (i >= sec->key_size) {
+		if (i >= vhost->key_size) {
 			seclog(sec, LOG_INFO,
-			       "received out-of-bounds key index (%d)", i);
+			       "%sreceived out-of-bounds key index (%d); have %d keys", PREFIX_VHOST(vhost), i, vhost->key_size);
 			return -1;
 		}
 
-		pkm->pk = gnutls_privkey_get_pk_algorithm(sec->key[i], NULL);
+		pkm->pk = gnutls_privkey_get_pk_algorithm(vhost->key[i], &bits);
+		pkm->bits = bits;
 
 		ret = send_msg(pool, cfd, CMD_SEC_GET_PK, pkm,
 			       (pack_size_func) sec_get_pk_msg__get_packed_size,
@@ -256,10 +260,13 @@ int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_
 			return -1;
 		}
 
+		vhost = find_vhost(sec->vconfig, op->vhost);
+		assert(vhost != NULL);
+
 		i = op->key_idx;
-		if (op->has_key_idx == 0 || i >= sec->key_size) {
+		if (op->has_key_idx == 0 || i >= vhost->key_size) {
 			seclog(sec, LOG_INFO,
-			       "received out-of-bounds key index (%d)", i);
+			       "%sreceived out-of-bounds key index (%d); have %d keys", PREFIX_VHOST(vhost), i, vhost->key_size);
 			return -1;
 		}
 
@@ -267,9 +274,9 @@ int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_
 		data.size = op->data.len;
 
 		if (cmd == CMD_SEC_SIGN_DATA) {
-			ret = gnutls_privkey_sign_data2(sec->key[i], op->sig, 0, &data, &out);
+			ret = gnutls_privkey_sign_data2(vhost->key[i], op->sig, 0, &data, &out);
 		} else {
-			ret = gnutls_privkey_sign_hash2(sec->key[i], op->sig, 0, &data, &out);
+			ret = gnutls_privkey_sign_hash2(vhost->key[i], op->sig, 0, &data, &out);
 		}
 		sec_op_msg__free_unpacked(op, &pa);
 
@@ -292,10 +299,13 @@ int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_
 			return -1;
 		}
 
+		vhost = find_vhost(sec->vconfig, op->vhost);
+		assert(vhost != NULL);
+
 		i = op->key_idx;
-		if (op->has_key_idx == 0 || i >= sec->key_size) {
+		if (op->has_key_idx == 0 || i >= vhost->key_size) {
 			seclog(sec, LOG_INFO,
-			       "received out-of-bounds key index (%d)", i);
+			       "%sreceived out-of-bounds key index (%d); have %d keys", PREFIX_VHOST(vhost), i, vhost->key_size);
 			return -1;
 		}
 
@@ -304,19 +314,13 @@ int process_worker_packet(void *pool, int cfd, pid_t pid, sec_mod_st * sec, cmd_
 
 		if (cmd == CMD_SEC_DECRYPT) {
 			ret =
-			    gnutls_privkey_decrypt_data(sec->key[i], 0, &data,
+			    gnutls_privkey_decrypt_data(vhost->key[i], 0, &data,
 							&out);
 		} else {
-#if GNUTLS_VERSION_NUMBER >= 0x030200
 			ret =
-			    gnutls_privkey_sign_hash(sec->key[i], 0,
+			    gnutls_privkey_sign_hash(vhost->key[i], 0,
 						     GNUTLS_PRIVKEY_SIGN_FLAG_TLS1_RSA,
 						     &data, &out);
-#else
-			ret =
-			    gnutls_privkey_sign_raw_data(sec->key[i], 0, &data,
-							 &out);
-#endif
 		}
 		sec_op_msg__free_unpacked(op, &pa);
 
@@ -494,6 +498,16 @@ int process_packet_from_main(void *pool, int fd, sec_mod_st * sec, cmd_request_t
 	data.size = buffer_size;
 
 	switch (cmd) {
+	case CMD_SECM_RELOAD:
+		reload_server(sec);
+
+		ret = send_msg(pool, fd, CMD_SECM_RELOAD_REPLY, NULL,
+			       NULL, NULL);
+		if (ret < 0) {
+			seclog(sec, LOG_ERR, "could not send reload reply to main!\n");
+			return ERR_BAD_COMMAND;
+		}
+		break;
 	case CMD_SECM_LIST_COOKIES:
 		handle_secm_list_cookies_reply(pool, fd, sec);
 
@@ -559,11 +573,6 @@ static void handle_alarm(int signo)
 	need_maintainance = 1;
 }
 
-static void handle_sighup(int signo)
-{
-	need_reload = 1;
-}
-
 static void handle_sigterm(int signo)
 {
 	need_exit = 1;
@@ -575,8 +584,8 @@ static void send_stats_to_main(sec_mod_st *sec)
 	time_t now = time(0);
 	SecmStatsMsg msg = SECM_STATS_MSG__INIT;
 
-	if (sec->perm_config->stats_reset_time != 0 &&
-	    now - sec->last_stats_reset > sec->perm_config->stats_reset_time) {
+	if (GETPCONFIG(sec)->stats_reset_time != 0 &&
+	    now - sec->last_stats_reset > GETPCONFIG(sec)->stats_reset_time) {
 		sec->auth_failures = 0;
 		sec->avg_auth_time = 0;
 		sec->max_auth_time = 0;
@@ -606,27 +615,46 @@ static void send_stats_to_main(sec_mod_st *sec)
 	return;
 }
 
+static void reload_server(sec_mod_st *sec)
+{
+	vhost_cfg_st *vhost = NULL;
+
+	seclog(sec, LOG_DEBUG, "reloading configuration");
+	reload_cfg_file(sec, sec->vconfig, 1);
+	load_keys(sec, 0);
+
+	list_for_each(sec->vconfig, vhost, list) {
+		sec_auth_init(vhost);
+	}
+	sup_config_init(sec);
+
+	need_reload = 0;
+}
+
 static void check_other_work(sec_mod_st *sec)
 {
+	vhost_cfg_st *vhost = NULL;
+
 	if (need_exit) {
 		unsigned i;
 
-		for (i = 0; i < sec->key_size; i++) {
-			gnutls_privkey_deinit(sec->key[i]);
+		list_for_each(sec->vconfig, vhost, list) {
+			for (i = 0; i < vhost->key_size; i++) {
+				gnutls_privkey_deinit(vhost->key[i]);
+				vhost->key[i] = NULL;
+			}
+			vhost->key_size = 0;
 		}
 
 		sec_mod_client_db_deinit(sec);
 		tls_cache_deinit(&sec->tls_db);
-		talloc_free(sec);
+		talloc_free(sec->config_pool);
+		talloc_free(sec->sec_mod_pool);
 		exit(0);
 	}
 
 	if (need_reload) {
-		seclog(sec, LOG_DEBUG, "reloading configuration");
-		reload_cfg_file(sec, sec->perm_config, 0);
-		sec->config = sec->perm_config->config;
-		load_keys(sec, 0);
-		need_reload = 0;
+		reload_server(sec);
 	}
 
 	if (need_maintainance) {
@@ -736,90 +764,96 @@ int serve_request_worker(sec_mod_st *sec, int cfd, pid_t pid, uint8_t *buffer, u
 #define CHECK_LOOP_ERR(x) \
 	if (force != 0) { GNUTLS_FATAL_ERR(x); } \
 	else { if (ret < 0) { \
-		seclog(sec, LOG_ERR, "could not reload key %s", sec->perm_config->key[i]); \
+		seclog(sec, LOG_ERR, "could not reload key %s", vhost->perm_config.key[i]); \
 		continue; } \
 	}
 
 static int load_keys(sec_mod_st *sec, unsigned force)
 {
-	unsigned i, need_reload = 0;
+	unsigned i, need_reload;
 	int ret;
-	struct pin_st pins;
-	static time_t last_access = 0;
+	vhost_cfg_st *vhost = NULL;
 
-	for (i = 0; i < sec->perm_config->key_size; i++) {
-		if (need_file_reload(sec->perm_config->key[i], last_access) != 0) {
-			need_reload = 1;
-			break;
-		}
-	}
+	list_for_each_rev(sec->vconfig, vhost, list) {
+		if (force == 0) {
+			need_reload = 0;
 
-	if (need_reload == 0)
-		return 0;
-
-	last_access = time(0);
-
-	ret = load_pins(sec->perm_config, &pins);
-	if (ret < 0) {
-		seclog(sec, LOG_ERR, "error loading PIN files");
-		exit(1);
-	}
-
-	/* Reminder: the number of private keys or their filenames cannot be changed on reload
-	 */
-	if (sec->key == NULL) {
-		sec->key_size = sec->perm_config->key_size;
-		sec->key = talloc_zero_size(sec, sizeof(*sec->key) * sec->perm_config->key_size);
-		if (sec->key == NULL) {
-			seclog(sec, LOG_ERR, "error in memory allocation");
-			exit(1);
-		}
-	}
-
-	/* read private keys */
-	for (i = 0; i < sec->key_size; i++) {
-		gnutls_privkey_t p;
-
-		ret = gnutls_privkey_init(&p);
-		CHECK_LOOP_ERR(ret);
-		/* load the private key */
-		if (gnutls_url_is_supported(sec->perm_config->key[i]) != 0) {
-			gnutls_privkey_set_pin_function(p,
-							pin_callback, &pins);
-			ret =
-			    gnutls_privkey_import_url(p,
-						      sec->perm_config->key[i], 0);
-			CHECK_LOOP_ERR(ret);
-		} else {
-			gnutls_datum_t data;
-			ret = gnutls_load_file(sec->perm_config->key[i], &data);
-			if (ret < 0) {
-				seclog(sec, LOG_ERR, "error loading file '%s'",
-				       sec->perm_config->key[i]);
-				CHECK_LOOP_ERR(ret);
+			for (i = 0; i < vhost->perm_config.key_size; i++) {
+				if (need_file_reload(vhost->perm_config.key[i], vhost->cert_last_access) != 0) {
+					need_reload = 1;
+					break;
+				}
 			}
 
-			ret =
-			    gnutls_privkey_import_x509_raw(p, &data,
-							   GNUTLS_X509_FMT_PEM,
-							   NULL, 0);
-			if (ret == GNUTLS_E_DECRYPTION_FAILED && pins.pin[0]) {
+			if (need_reload == 0)
+				continue;
+		}
+
+		vhost->cert_last_access = time(0);
+
+		ret = load_pins(GETPCONFIG(sec), &vhost->pins);
+		if (ret < 0) {
+			seclog(sec, LOG_ERR, "error loading PIN files");
+			exit(1);
+		}
+
+		/* Reminder: the number of private keys or their filenames cannot be changed on reload
+		 */
+		if (vhost->key == NULL) {
+			vhost->key_size = vhost->perm_config.key_size;
+			vhost->key = talloc_zero_size(sec, sizeof(*vhost->key) * vhost->perm_config.key_size);
+			if (vhost->key == NULL) {
+				seclog(sec, LOG_ERR, "error in memory allocation");
+				exit(1);
+			}
+		}
+
+		/* read private keys */
+		for (i = 0; i < vhost->key_size; i++) {
+			gnutls_privkey_t p;
+
+			ret = gnutls_privkey_init(&p);
+			CHECK_LOOP_ERR(ret);
+
+			/* load the private key */
+			if (gnutls_url_is_supported(vhost->perm_config.key[i]) != 0) {
+				gnutls_privkey_set_pin_function(p,
+								pin_callback, &vhost->pins);
+				ret =
+				    gnutls_privkey_import_url(p,
+							      vhost->perm_config.key[i], 0);
+				CHECK_LOOP_ERR(ret);
+			} else {
+				gnutls_datum_t data;
+				ret = gnutls_load_file(vhost->perm_config.key[i], &data);
+				if (ret < 0) {
+					seclog(sec, LOG_ERR, "error loading file '%s'",
+					       vhost->perm_config.key[i]);
+					CHECK_LOOP_ERR(ret);
+				}
+
 				ret =
 				    gnutls_privkey_import_x509_raw(p, &data,
 								   GNUTLS_X509_FMT_PEM,
-								   pins.pin, 0);
+								   NULL, 0);
+				if (ret == GNUTLS_E_DECRYPTION_FAILED && vhost->pins.pin[0]) {
+					ret =
+					    gnutls_privkey_import_x509_raw(p, &data,
+									   GNUTLS_X509_FMT_PEM,
+									   vhost->pins.pin, 0);
+				}
+				CHECK_LOOP_ERR(ret);
+
+				gnutls_free(data.data);
 			}
-			CHECK_LOOP_ERR(ret);
 
-			gnutls_free(data.data);
+			if (vhost->key[i] != NULL) {
+				gnutls_privkey_deinit(vhost->key[i]);
+			}
+			vhost->key[i] = p;
 		}
-
-		if (sec->key[i] != NULL) {
-			gnutls_privkey_deinit(sec->key[i]);
-		}
-		sec->key[i] = p;
+		seclog(sec, LOG_DEBUG, "%sloaded %d keys\n", PREFIX_VHOST(vhost), vhost->key_size);
 	}
-
 	return 0;
 }
 
@@ -852,8 +886,8 @@ static int load_keys(sec_mod_st *sec, unsigned force)
  * clients fast without becoming a bottleneck due to private 
  * key operations.
  */
-void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char *socket_file,
-		    int cmd_fd, int cmd_fd_sync)
+void sec_mod_server(void *main_pool, void *config_pool, struct list_head *vconfig,
+		    const char *socket_file, int cmd_fd, int cmd_fd_sync)
 {
 	struct sockaddr_un sa;
 	socklen_t sa_len;
@@ -864,6 +898,7 @@ void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char
 	int sd;
 	sec_mod_st *sec;
 	void *sec_mod_pool;
+	vhost_cfg_st *vhost = NULL;
 	fd_set rd_set;
 	pid_t pid;
 #ifdef HAVE_PSELECT
@@ -895,8 +930,9 @@ void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char
 		exit(1);
 	}
 
-	sec->perm_config = talloc_steal(sec, perm_config);
-	sec->config = sec->perm_config->config;
+	sec->vconfig = vconfig;
+	sec->config_pool = config_pool;
+	sec->sec_mod_pool = sec_mod_pool;
 
 	tls_cache_init(sec, &sec->tls_db);
 	sup_config_init(sec);
@@ -911,23 +947,18 @@ void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char
 	/* we no longer need the main pool after this point. */
 	talloc_free(main_pool);
 
-	ocsignal(SIGHUP, handle_sighup);
+	ocsignal(SIGHUP, SIG_IGN);
 	ocsignal(SIGINT, handle_sigterm);
 	ocsignal(SIGTERM, handle_sigterm);
 	ocsignal(SIGALRM, handle_alarm);
 
-	sec_auth_init(sec, perm_config);
+	list_for_each(sec->vconfig, vhost, list) {
+		sec_auth_init(vhost);
+	}
+
 	sec->cmd_fd = cmd_fd;
 	sec->cmd_fd_sync = cmd_fd_sync;
 
-#ifdef HAVE_PKCS11
-	ret = gnutls_pkcs11_reinit();
-	if (ret < 0) {
-		seclog(sec, LOG_WARNING, "error in PKCS #11 reinitialization: %s",
-		       gnutls_strerror(ret));
-	}
-#endif
-
 	if (sec_mod_client_db_init(sec) == NULL) {
 		seclog(sec, LOG_ERR, "error in client db initialization");
 		exit(1);
@@ -951,7 +982,7 @@ void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char
 		exit(1);
 	}
 
-	ret = chown(SOCKET_FILE, perm_config->uid, perm_config->gid);
+	ret = chown(SOCKET_FILE, GETPCONFIG(sec)->uid, GETPCONFIG(sec)->gid);
 	if (ret == -1) {
 		e = errno;
 		seclog(sec, LOG_INFO, "could not chown socket '%s': %s", SOCKET_FILE,
@@ -1057,7 +1088,9 @@ void sec_mod_server(void *main_pool, struct perm_cfg_st *perm_config, const char
 
 			/* do not allow unauthorized processes to issue commands
 			 */
-			ret = check_upeer_id("sec-mod", sec->perm_config->debug, cfd, perm_config->uid, perm_config->gid, &uid, &pid);
+			ret = check_upeer_id("sec-mod", GETPCONFIG(sec)->debug, cfd,
+					     GETPCONFIG(sec)->uid, GETPCONFIG(sec)->gid,
+					     &uid, &pid);
 			if (ret < 0) {
 				seclog(sec, LOG_INFO, "rejected unauthorized connection");
 			} else {
