@@ -35,6 +35,7 @@
 #include <errno.h>
 #include <cloexec.h>
 #include <ip-lease.h>
+#include <minmax.h>
 
 #if defined(HAVE_LINUX_IF_TUN_H)
 # include <linux/if_tun.h>
@@ -47,6 +48,7 @@
 #include <tun.h>
 #include <main.h>
 #include <ccan/list/list.h>
+#include "vhost.h"
 
 #if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
 # include <net/if_var.h>
@@ -71,7 +73,7 @@ struct in6_ifreq {
 };
 
 static
-int set_ipv6_addr(main_server_st * s, struct proc_st *proc)
+int os_set_ipv6_addr(main_server_st * s, struct proc_st *proc)
 {
 	int fd, e, ret;
 	struct in6_ifreq ifr6;
@@ -158,7 +160,7 @@ int set_ipv6_addr(main_server_st * s, struct proc_st *proc)
 	return ret;
 }
 
-static void reset_ipv6_addr(struct proc_st *proc)
+static void os_reset_ipv6_addr(struct proc_st *proc)
 {
 	int fd, ret;
 	struct in6_ifreq ifr6;
@@ -217,7 +219,7 @@ static void reset_ipv6_addr(struct proc_st *proc)
 #include <netinet6/nd6.h>
 
 static
-int set_ipv6_addr(main_server_st * s, struct proc_st *proc)
+int os_set_ipv6_addr(main_server_st * s, struct proc_st *proc)
 {
 	int fd, e, ret;
 	struct in6_aliasreq ifr6;
@@ -285,7 +287,7 @@ int set_ipv6_addr(main_server_st * s, struct proc_st *proc)
 	return ret;
 }
 
-static void reset_ipv6_addr(struct proc_st *proc)
+static void os_reset_ipv6_addr(struct proc_st *proc)
 {
 	struct in6_ifreq ifr6;
 	int fd;
@@ -311,12 +313,12 @@ static void reset_ipv6_addr(struct proc_st *proc)
 
 #else
 #warning "No IPv6 support on this platform"
-static int set_ipv6_addr(main_server_st * s, struct proc_st *proc)
+static int os_set_ipv6_addr(main_server_st * s, struct proc_st *proc)
 {
 	return -1;
 }
 
-static void reset_ipv6_addr(struct proc_st *proc)
+static void os_reset_ipv6_addr(struct proc_st *proc)
 {
 	return;
 }
@@ -412,7 +414,7 @@ static int set_network_info(main_server_st * s, struct proc_st *proc)
 	}
 
 	if (proc->ipv6 && proc->ipv6->lip_len > 0 && proc->ipv6->rip_len > 0) {
-		ret = set_ipv6_addr(s, proc);
+		ret = os_set_ipv6_addr(s, proc);
 		if (ret < 0) {
 			remove_ip_lease(s, proc->ipv6);
 			proc->ipv6 = NULL;
@@ -436,14 +438,89 @@ static int set_network_info(main_server_st * s, struct proc_st *proc)
 
 #include <ccan/hash/hash.h>
 
-#ifndef __linux__
-static int bsd_open_tun(main_server_st * s)
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+
+static int bsd_ifrename(main_server_st *s, struct proc_st *proc)
 {
-	int fd, e;
+#ifdef SIOCSIFNAME
+	int fd = -1;
+	int e, ret;
+	struct ifreq ifr;
+	uint8_t ctr;
+	unsigned i;
+	char tun_name[IFNAMSIZ];
+	static unsigned next_tun_nr = 0;
+	unsigned renamed = 0;
+
+	fd = socket(AF_INET, SOCK_DGRAM, 0);
+	if (fd == -1)
+		return -1;
+
+	memset(&ifr, 0, sizeof(struct ifreq));
+	strlcpy(ifr.ifr_name, proc->tun_lease.name, IFNAMSIZ);
+
+	ret = snprintf(tun_name, sizeof(tun_name), "%s%u",
+		       GETCONFIG(s)->network.name, next_tun_nr+1024);
+	if (ret >= sizeof(tun_name))
+		next_tun_nr = 0;
+
+	ctr = next_tun_nr;
+
+	for (i=ctr;i<ctr+1024;i++) {
+		ret = snprintf(tun_name, sizeof(tun_name), "%s%u",
+			       GETCONFIG(s)->network.name, i);
+		if (ret != strlen(tun_name)) {
+			mslog(s, NULL, LOG_ERR, "Truncation error in tun name: %s; adjust 'device' option\n",
+			      proc->tun_lease.name);
+			return -1;
+		}
+
+		ifr.ifr_data = tun_name;
+
+		ret = ioctl(fd, SIOCSIFNAME, &ifr);
+		if (ret != 0) {
+			e = errno;
+			if (e == EEXIST)
+				continue;
+
+			mslog(s, NULL, LOG_ERR, "%s: Error renaming interface: %s\n",
+				proc->tun_lease.name, strerror(e));
+			goto fail;
+		}
+
+		renamed = 1;
+		break;
+	}
+
+
+	/* set new name */
+	next_tun_nr = ctr+1;
+
+	if (renamed) {
+		strlcpy(proc->tun_lease.name, tun_name, sizeof(proc->tun_lease.name));
+		ret = 0;
+	} else {
+		e = errno;
+		mslog(s, NULL, LOG_WARNING, "Error renaming interface: %s to %s: %s\n",
+		      proc->tun_lease.name, tun_name, strerror(e));
+		ret = -1;
+	}
+
+ fail:
+	close(fd);
+
+	return ret;
+#endif
+}
+
+/* BSD version */
+static int os_open_tun(main_server_st * s, struct proc_st *proc)
+{
+	int fd, e, ret;
 	int sock;
-	char tun_name[80];
 	struct ifreq ifr;
 	int unit_nr = 0;
+	struct stat st;
 
 	fd = open("/dev/tun", O_RDWR);
 	if (fd == -1) {
@@ -451,9 +528,9 @@ static int bsd_open_tun(main_server_st * s)
 		e = errno;
 		mslog(s, NULL, LOG_DEBUG, "cannot open /dev/tun; falling back to iteration: %s", strerror(e));
 		for (unit_nr = 0; unit_nr < 255; unit_nr++) {
-			snprintf(tun_name, sizeof(tun_name), "/dev/tun%d", unit_nr);
-			fd = open(tun_name, O_RDWR);
-# ifdef SIOCIFCREATE
+			snprintf(proc->tun_lease.name, sizeof(proc->tun_lease.name), "/dev/tun%d", unit_nr);
+			fd = open(proc->tun_lease.name, O_RDWR);
+#ifdef SIOCIFCREATE
 			if (fd == -1) {
 				/* cannot open tunXX, try creating it */
 				sock = socket(AF_INET, SOCK_DGRAM, 0);
@@ -464,17 +541,30 @@ static int bsd_open_tun(main_server_st * s)
 				}
 
 				memset(&ifr, 0, sizeof(ifr));
-				strncpy(ifr.ifr_name, tun_name + 5, sizeof(ifr.ifr_name) - 1);
+				strncpy(ifr.ifr_name, proc->tun_lease.name + 5, sizeof(ifr.ifr_name) - 1);
 				if (!ioctl(sock, SIOCIFCREATE, &ifr))
-					fd = open(tun_name, O_RDWR);
+					fd = open(proc->tun_lease.name, O_RDWR);
 				close(sock);
 			}
-# endif
+#endif
 			if (fd >= 0)
 				break;
 		}
 	}
 
+	if (fd < 0)
+		return fd;
+
+	/* get tun name */
+	ret = fstat(fd, &st);
+	if (ret < 0) {
+		e = errno;
+		mslog(s, NULL, LOG_ERR, "tun fd %d: stat: %s\n", fd, strerror(e));
+		close(fd);
+		return -1;
+	}
+	strlcpy(proc->tun_lease.name, devname(st.st_rdev, S_IFCHR), sizeof(proc->tun_lease.name));
+
 	if (fd >= 0) {
 		int i, e, ret;
 #if defined(__OpenBSD__)
@@ -484,7 +574,7 @@ static int bsd_open_tun(main_server_st * s)
 		if (ret < 0) {
 			e = errno;
 			mslog(s, NULL, LOG_ERR, "%s: TUNGIFINFO: %s\n",
-					tun_name, strerror(e));
+					proc->tun_lease.name, strerror(e));
 		} else {
 			inf.flags |= IFF_MULTICAST;
 
@@ -492,7 +582,7 @@ static int bsd_open_tun(main_server_st * s)
 			if (ret < 0) {
 				e = errno;
 				mslog(s, NULL, LOG_ERR, "%s: TUNSIFINFO: %s\n",
-						tun_name, strerror(e));
+						proc->tun_lease.name, strerror(e));
 			}
 		}
 #else /* FreeBSD + NetBSD */
@@ -501,7 +591,7 @@ static int bsd_open_tun(main_server_st * s)
 		if (ret < 0) {
 			e = errno;
 			mslog(s, NULL, LOG_ERR, "%s: TUNSIFMODE: %s\n",
-			      tun_name, strerror(e));
+			      proc->tun_lease.name, strerror(e));
 		}
 
 		/* link layer mode off */
@@ -510,7 +600,7 @@ static int bsd_open_tun(main_server_st * s)
 		if (ret < 0) {
 			e = errno;
 			mslog(s, NULL, LOG_ERR, "%s: TUNSLMODE: %s\n",
-			      tun_name, strerror(e));
+			      proc->tun_lease.name, strerror(e));
 		}
 #endif
 
@@ -521,38 +611,37 @@ static int bsd_open_tun(main_server_st * s)
 		if (ret < 0) {
 			e = errno;
 			mslog(s, NULL, LOG_ERR, "%s: TUNSIFHEAD: %s\n",
-			      tun_name, strerror(e));
+			      proc->tun_lease.name, strerror(e));
 		}
 #endif /* TUNSIFHEAD */
 
 	}
 
+	/* rename the device if possible */
+	if (bsd_ifrename(s, proc) < 0) {
+		close(fd);
+		return -1;
+	}
+
 	return fd;
 }
-#endif
-
-int open_tun(main_server_st * s, struct proc_st *proc)
+#elif defined(__linux__)
+/* Linux version */
+static int os_open_tun(main_server_st * s, struct proc_st *proc)
 {
 	int tunfd, ret, e;
 	struct ifreq ifr;
 	unsigned int t;
 
-	ret = get_ip_leases(s, proc);
-	if (ret < 0)
-		return ret;
 	ret = snprintf(proc->tun_lease.name, sizeof(proc->tun_lease.name), "%s%%d",
-		       s->config->network.name);
+		       GETCONFIG(s)->network.name);
 	if (ret != strlen(proc->tun_lease.name)) {
 		mslog(s, NULL, LOG_ERR, "Truncation error in tun name: %s; adjust 'device' option\n",
 		      proc->tun_lease.name);
 		return -1;
 	}
 
-	/* No need to free the lease after this point.
-	 */
-
 	/* Obtain a free tun device */
-#ifdef __linux__
 	tunfd = open("/dev/net/tun", O_RDWR);
 	if (tunfd < 0) {
 		int e = errno;
@@ -561,8 +650,6 @@ int open_tun(main_server_st * s, struct proc_st *proc)
 		return -1;
 	}
 
-	set_cloexec_flag(tunfd, 1);
-
 	memset(&ifr, 0, sizeof(ifr));
 	ifr.ifr_flags = IFF_TUN | IFF_NO_PI;
 
@@ -586,8 +673,8 @@ int open_tun(main_server_st * s, struct proc_st *proc)
 		goto fail;
 	}
 
-	if (s->perm_config->uid != -1) {
-		t = s->perm_config->uid;
+	if (GETPCONFIG(s)->uid != -1) {
+		t = GETPCONFIG(s)->uid;
 		ret = ioctl(tunfd, TUNSETOWNER, t);
 		if (ret < 0) {
 			e = errno;
@@ -597,8 +684,8 @@ int open_tun(main_server_st * s, struct proc_st *proc)
 		}
 	}
 #ifdef TUNSETGROUP
-	if (s->perm_config->gid != -1) {
-		t = s->perm_config->gid;
+	if (GETPCONFIG(s)->gid != -1) {
+		t = GETPCONFIG(s)->gid;
 		ret = ioctl(tunfd, TUNSETGROUP, t);
 		if (ret < 0) {
 			e = errno;
@@ -612,31 +699,33 @@ int open_tun(main_server_st * s, struct proc_st *proc)
 		}
 	}
 #endif
-#else				/* freebsd */
-	tunfd = bsd_open_tun(s);
+
+	return tunfd;
+ fail:
+	close(tunfd);
+	return -1;
+}
+#endif /* __linux__ */
+
+int open_tun(main_server_st * s, struct proc_st *proc)
+{
+	int tunfd, ret;
+
+	ret = get_ip_leases(s, proc);
+	if (ret < 0)
+		return ret;
+
+	/* No need to free the lease after this point.
+	 */
+	tunfd = os_open_tun(s, proc);
 	if (tunfd < 0) {
 		int e = errno;
-		mslog(s, NULL, LOG_ERR, "Can't open /dev/tun: %s\n",
+		mslog(s, NULL, LOG_ERR, "Can't open tun device: %s\n",
 		      strerror(e));
 		return -1;
 	}
 
-	/* find device name */
-	{
-		struct stat st;
-
-		ret = fstat(tunfd, &st);
-		if (ret < 0) {
-			e = errno;
-			mslog(s, NULL, LOG_ERR, "tun fd %d: stat: %s\n", tunfd, strerror(e));
-			goto fail;
-		}
-
-		strlcpy(proc->tun_lease.name, devname(st.st_rdev, S_IFCHR), sizeof(proc->tun_lease.name));
-	}
-
 	set_cloexec_flag(tunfd, 1);
-#endif
 
 	if (proc->tun_lease.name[0] == 0) {
 		mslog(s, NULL, LOG_ERR, "tun device with no name!");
@@ -736,16 +825,17 @@ void reset_tun(struct proc_st* proc)
 {
 	if (proc->tun_lease.name[0] != 0) {
 		reset_ipv4_addr(proc);
-		reset_ipv6_addr(proc);
+		os_reset_ipv6_addr(proc);
 	}
 }
 
 #if defined(__OpenBSD__) || defined(TUNSIFHEAD)
 # define TUN_AF_PREFIX 1
 #endif
+
+#ifdef TUN_AF_PREFIX
 ssize_t tun_write(int sockfd, const void *buf, size_t len)
 {
-#ifdef TUN_AF_PREFIX
 	struct ip *iph = (void *)buf;
 	uint32_t head;
 	const uint8_t *data = buf;
@@ -775,14 +865,10 @@ ssize_t tun_write(int sockfd, const void *buf, size_t len)
 	if (ret >= sizeof(uint32_t))
 		ret -= sizeof(uint32_t);
 	return ret;
-#else
-	return force_write(sockfd, buf, len);
-#endif
 }
 
 ssize_t tun_read(int sockfd, void *buf, size_t len)
 {
-#ifdef TUN_AF_PREFIX
 	uint32_t head;
 	struct iovec iov[2];
 	int ret;
@@ -796,7 +882,16 @@ ssize_t tun_read(int sockfd, void *buf, size_t len)
 	if (ret >= sizeof(uint32_t))
 		ret -= sizeof(uint32_t);
 	return ret;
-#else
-	return read(sockfd, buf, len);
-#endif
 }
+
+#else
+ssize_t tun_write(int sockfd, const void *buf, size_t len)
+{
+	return force_write(sockfd, buf, len);
+}
+
+ssize_t tun_read(int sockfd, void *buf, size_t len)
+{
+	return read(sockfd, buf, len);
+}
+#endif
